@@ -7,14 +7,15 @@ import psutil
 import uuid
 from functools import partial
 from concurrent.futures import ProcessPoolExecutor
+import scipy.stats as st
 from plotnine import ggplot, aes, geom_point, geom_errorbarh, geom_errorbar, theme, element_text, geom_abline, labs, expand_limits
 
 
-from .proxy import find_proxies, apply_proxies, query_outcome_proxy
-from .MR_tools import query_outcome_func, harmonize_MR, MR_func, mrpresso_func
-from .clump import clump_data
+from .proxy import find_proxies, apply_proxies
+from .MR_tools import query_outcome_func, MR_func, mrpresso_func
+from .clump import clump_data_plink2
 from .lift import lift_data
-from .tools import create_tmp, get_plink19_path, load_reference_panel, setup_genetic_path
+from .tools import create_tmp, load_reference_panel, setup_genetic_path
 from .geno_tools import (
     save_data,
     check_arguments,
@@ -31,7 +32,7 @@ from .geno_tools import (
     check_snp_column,
     remove_na
 )
-from .association import set_phenotype_func, association_test_func
+from .association import set_phenotype_func, association_test_func_plink2
 from .extract_prs import extract_snps_func, prs_func
 from .snp_query import async_query_gwas_catalog
 from .constants import STANDARD_COLUMNS, REF_PANEL_COLUMNS, CHECKS_DICT, MR_METHODS_NAMES
@@ -41,9 +42,7 @@ from .constants import STANDARD_COLUMNS, REF_PANEL_COLUMNS, CHECKS_DICT, MR_METH
 # Add proxying function (input is df + searchspace (list of SNP or path to .bim, can be separated by chromosomes) and returns proxied df)
 # Get proxies (simply return a list of proxies)
 # Multi-MR with python MR
-# Switch to plink2.0 (but check that all functions exist)
-# Handle genetic files in pgen (requires plink2.0)
-# Helper function to read bim or pvar (or other variant files) from a genetic path and handle the cases where the chr column is not only a number
+# Check stability with variants on sexual chromosomes
 
 
 class Geno:
@@ -169,7 +168,7 @@ class Geno:
                 - "Fill": Missing columns are added based on reference data and invalid values set to NaN, but no rows are deleted.
                 - "Fill_delete": Missing columns are added, and rows with missing, duplicated, or invalid values are deleted.
                 Defaults to 'Fill'.
-            reference_panel (str or pd.DataFrame, optional): Reference panel for SNP adjustments. Can be a string representing ancestry classification ("eur", "afr", "eas", "sas", "amr") or a DataFrame with ["CHR","SNP","POS","A1","A2"] columns or a path to a .bim file. Defaults to "eur".
+            reference_panel (str or pd.DataFrame, optional): Reference panel for SNP adjustments. Can be a string representing ancestry classification ("eur", "afr", "eas", "sas", "amr") or a DataFrame with ["CHR","SNP","POS","A1","A2"] columns or a path to a .bim or .pvar file. Defaults to "eur".
             effect_column (str, optional): Specifies the type of effect column ("BETA" or "OR"). If None, the method tries to determine it. Odds Ratios will be log-transformed and the standard error adjusted. Defaults to None.
             keep_multi (bool, optional): Determines if multiallelic SNPs should be kept. If None, defers to preprocessing value. Defaults to None.
             keep_dups (bool, optional): Determines if rows with duplicate SNP IDs should be kept. If None, defers to preprocessing value. Defaults to None.
@@ -182,7 +181,6 @@ class Geno:
         # Check arguments and solve arguments logic.
         keep_multi, keep_dups, fill_snpids, fill_coordinates = check_arguments(
             preprocessing,
-            reference_panel,
             effect_column,
             fill_snpids,
             fill_coordinates,
@@ -266,11 +264,9 @@ class Geno:
             self.checks["SNP"] = True
 
         # Warn if essential columns are missing
-        for column in STANDARD_COLUMNS:
-            if not (column in data.columns):
-                print(
-                    f"Warning: the data doesn't include a {column} column. This may become an issue later on."
-                )
+        missing_columns = [column for column in STANDARD_COLUMNS if column not in data.columns]
+        if missing_columns:
+            print(f"Warning: the data doesn't include the following columns: {', '.join(missing_columns)}. Some methods may need them.")
 
         # Remove missing values if preprocessing level is set to 'Fill_delete'
         if preprocessing == 'Fill_delete':
@@ -289,7 +285,7 @@ class Geno:
         If the Geno object does not have a reference panel attribute set,
         this method will try to set it based on the provided `reference_panel`
         argument. This can be either a string indicating a predefined reference panel
-        or a DataFrame with specific columns or a path to a .bim file.
+        or a DataFrame with specific columns or a path to a .bim or .pvar file.
 
         Args:
             reference_panel (str or pd.DataFrame, optional): Either a string indicating a predefined
@@ -335,7 +331,7 @@ class Geno:
             r2 (float, optional): Linkage disequilibrium threshold, values between 0 and 1. Default is 0.1.
             p1 (float, optional): P-value threshold during clumping. SNPs with a P-value higher than this value are excluded. Default is 5e-8.
             p2 (float, optional): P-value threshold post-clumping to further filter the clumped SNPs. If p2 < p1, it won't be considered. Default is 0.01.
-            reference_panel (str, optional): The reference population for linkage disequilibrium values. Accepts values "eur", "sas", "afr", "eas", "amr". Alternatively, a path leading to a specific bed/bim/fam reference panel can be provided. Default is "eur".
+            reference_panel (str, optional): The reference population for linkage disequilibrium values. Accepts values "eur", "sas", "afr", "eas", "amr". Alternatively, a path leading to a specific bed/bim/fam or pgen/pvar/psam reference panel can be provided. Default is "eur".
 
         Returns:
             genal.Geno: A new Geno object based on the clumped data.
@@ -369,7 +365,7 @@ class Geno:
         create_tmp()
 
         # Clump the data using the specified parameters
-        clumped_data = clump_data(
+        clumped_data = clump_data_plink2(
             self.data,
             reference_panel,
             kb,
@@ -394,7 +390,7 @@ class Geno:
         Update or create the column of SNP name based on genetic data and genomic position.
         
         Args:
-            path (str, optional): Path to a bed/bim/fam set of genetic files.
+            path (str, optional): Path to a bed/bim/fam or pgen/pvar/psam set of genetic files.
                 If files are split by chromosomes, replace the chromosome number with '$'.
                 For instance: path = "ukb_chr$_file". Defaults to the path from the configuration.
             replace (bool, optional): To update the .data attribute with the updated SNP column or not.
@@ -414,37 +410,48 @@ class Geno:
                     f"The column {column} is not found in the data and is mandatory to update snpIDs!"
                 )
         data = self.data.copy() # We don't want to modify the data attribute
-        path = setup_genetic_path(path) #Verify the path
-        filetype = "split" if "$" in path else "combined" #If data is split by chromosomes or not
-        # Merge data with the bim dataframe
-        if filetype == "combined":
-            bim = pd.read_csv(
-                path + ".bim", sep="\t", names=["CHR", "SNP", "F", "POS", "A1", "A2"]
+        path, filetype = setup_genetic_path(path) #Verify the path and get filetype
+        is_split = "$" in path #If data is split by chromosomes or not
+        
+        # Merge data with the variant info dataframe
+        if not is_split:
+            if filetype == "bed":
+                variant_info = pd.read_csv(
+                    path + ".bim", sep="\t", names=["CHR", "SNP", "F", "POS", "A1", "A2"]
+                )
+            else: # pgen
+                variant_info = pd.read_csv(
+                    path + ".pvar", sep="\t", comment="#", names=["CHR", "POS", "SNP", "A1", "A2"]
+                )
+
+            # Convert CHR to string and remove 'chr' prefix if present
+            variant_info["CHR"] = variant_info["CHR"].astype(str).str.replace("^chr", "", regex=True)
+            # Convert numeric values to int, leave X/Y as strings
+            variant_info["CHR"] = variant_info["CHR"].apply(
+                lambda x: int(float(x)) if x.replace(".", "").isdigit() else x
             )
 
-            # Convert CHR to string and remove 'chr' prefix if present, then convert to int
-            if str(bim["CHR"][0]).startswith("chr"):
-                bim["CHR"] = bim["CHR"].astype(str).str.replace("^chr", "", regex=True).astype(int)
-
             data = data.merge(
-                bim[["CHR", "POS", "SNP"]], on=["CHR", "POS"], how="left", suffixes=('', '_new')
+                variant_info[["CHR", "POS", "SNP"]], on=["CHR", "POS"], how="left", suffixes=('', '_new')
             )
         else:
             chr_dict = {k: v for k, v in data.groupby('CHR')} #Split the dataframe by chromosome
             partial_merge_command_parallel = partial(
-                merge_command_parallel, path=path
+                merge_command_parallel, path=path, filetype=filetype
             )  # Wrapper function
             with ProcessPoolExecutor() as executor: #Merge each dataframe subset
                 results = list(executor.map(partial_merge_command_parallel, chr_dict.values()))
                 data = pd.concat(results, ignore_index=True, axis=0) #And combine them again
+                
         # Update the SNP column
         data['SNP'] = data['SNP_new'].fillna(data['SNP'])
-        n_absent = data['SNP_new'].isna().sum()
-        data.drop(columns = ['SNP_new'], inplace=True)
+        #n_absent = data['SNP_new'].isna().sum()
         #if n_absent > 0:
         #    print(f"{n_absent}({n_absent/data.shape[0]*100:.3f}%) are not present in the genetic data.")
         #else:
         #    print("All SNPs are present in the genetic data.")
+        data.drop(columns = ['SNP_new'], inplace=True)
+        
         if replace: self.data = data #Update attribute if replace argument
         return data
     
@@ -453,12 +460,12 @@ class Geno:
         Extract the list of SNPs of this Geno object from the genetic data provided.
 
         Args:
-            path (str, optional): Path to a bed/bim/fam set of genetic files.
+            path (str, optional): Path to a bed/bim/fam or pgen/pvar/psam set of genetic files.
                 If files are split by chromosomes, replace the chromosome number with '$'.
                 For instance: path = "ukb_chr$_file". Default is None.
 
         Returns:
-            None: The output is a bed/bim/fam triple in the tmp_GENAL folder
+            None: The output is a pgen/pvar/psam triple in the tmp_GENAL folder
             with the format "{name}_extract_allchr" which includes the SNPs from the UKB.
 
         Notes:
@@ -493,7 +500,7 @@ class Geno:
             name (str, optional): Name or path of the saved PRS file.
             weighted (bool, optional): If True, performs a PRS weighted by the BETA column estimates.
                                        If False, performs an unweighted PRS. Default is True.
-            path (str, optional): Path to a bed/bim/fam set of genetic files for PRS calculation.
+            path (str, optional): Path to a bed/bim/fam or pgen/pvar/psam set of genetic files for PRS calculation.
                                   If files are split by chromosomes, replace the chromosome number
                                   with '$'. For instance: path = "ukb_chr$_file".
                                   If not provided, it will use the genetic path most recently used 
@@ -503,7 +510,7 @@ class Geno:
             proxy (bool, optional): If true, proxies are searched. Default is True.
             reference_panel (str, optional): The reference population used to derive linkage
                 disequilibrium values and find proxies (only if proxy=True). Acceptable values
-                include "EUR", "SAS", "AFR", "EAS", "AMR" or a path to a specific bed/bim/fam panel.
+                include "EUR", "SAS", "AFR", "EAS", "AMR" or a path to a specific bed/bim/fam or pgen/pvar/psam panel.
                 Default is "EUR".
             kb (int, optional): Width of the genomic window to look for proxies. Default is 5000.
             r2 (float, optional): Minimum linkage disequilibrium value with the main SNP
@@ -513,12 +520,9 @@ class Geno:
 
         Returns:
             pd.DataFrame: The computed PRS data.
-
-        Raises:
-            ValueError: If the data hasn't been clumped and 'clumped' parameter is True.
         """
         
-        path = setup_genetic_path(path) # Check path
+        path, filetype = setup_genetic_path(path) # Check path
         create_tmp() #Make sure temporary folder exists
         
         # Check for mandatory columns in data
@@ -557,35 +561,61 @@ class Geno:
         if proxy:
             print("Identifying the SNPs present in the genetic data...")
             # Obtain the list of SNPs present in the genetic data
-            if path.count("$") == 1: #If split: merge all SNP columns of the .bim files
+            if path.count("$") == 1: #If split: merge all SNP columns of the variant files in parallel
                 genetic_snp_list = []
                 for i in range(1,23):
-                    path_i = path.replace("$", str(i))+ ".bim"
-                    if os.path.exists(path_i):
-                        bim_i = pd.read_csv(
-                            path_i, sep="\t", names=["CHR", "SNP", "F", "POS", "A1", "A2"], usecols=["SNP"]
-                        )
-                        genetic_snp_list.extend(bim_i.SNP.tolist())
+                    path_i = path.replace("$", str(i))
+                    if filetype == "bed":
+                        variant_file = path_i + ".bim"
+                        if os.path.exists(variant_file):
+                            variants = pd.read_csv(
+                                variant_file, sep="\t", 
+                                names=["CHR", "SNP", "F", "POS", "A1", "A2"], 
+                                usecols=["SNP"]
+                            )
+                            genetic_snp_list.extend(variants.SNP.tolist())
+                    else: # filetype == "pgen"
+                        variant_file = path_i + ".pvar"
+                        if os.path.exists(variant_file):
+                            variants = pd.read_csv(
+                                variant_file, sep="\t", comment="#",
+                                names=["CHR", "POS", "SNP", "A1", "A2"],
+                                usecols=["SNP"]
+                            )
+                            genetic_snp_list.extend(variants.SNP.tolist())
             else: #If not split
-                bim_path = path + ".bim"
-                if os.path.exists(bim_path):
-                    bim = pd.read_csv(
-                            bim_path, 
-                            sep="\t", names=["CHR", "SNP", "F", "POS", "A1", "A2"]
+                if filetype == "bed":
+                    variant_file = path + ".bim"
+                    if os.path.exists(variant_file):
+                        variants = pd.read_csv(
+                            variant_file, sep="\t",
+                            names=["CHR", "SNP", "F", "POS", "A1", "A2"]
                         )
-                else:
-                    raise ValueError(f"The file {bim_path} does not exist.")
-                genetic_snp_list = bim.SNP.tolist()
+                    else:
+                        raise ValueError(f"The file {variant_file} does not exist.")
+                else: # filetype == "pgen"
+                    variant_file = path + ".pvar"
+                    if os.path.exists(variant_file):
+                        variants = pd.read_csv(
+                            variant_file, sep="\t", comment="#",
+                            names=["CHR", "POS", "SNP", "A1", "A2"]
+                        )
+                    else:
+                        raise ValueError(f"The file {variant_file} does not exist.")
+                genetic_snp_list = variants.SNP.tolist()
+                
             # Identify the SNPs already present in the genetic data
             genetic_snps = set(genetic_snp_list)
             exposure_snps = set(data_prs.SNP.values)
-            snps_present = exposure_snps & genetic_snps
+            snps_present = len(exposure_snps & genetic_snps)
+            snps_absent = exposure_snps - genetic_snps
+
             print(
-                f"{len(snps_present)} SNPs out of {len(exposure_snps)} are present in the genetic data."
+                f"{snps_present} SNPs out of {len(exposure_snps)} are present in the genetic data."
             )
-            # Search proxies for absent SNPs
-            if len(exposure_snps) - len(snps_present) > 0:
-                snps_absent = exposure_snps - snps_present
+
+            # Search proxies only if there are absent SNPs
+            if snps_absent:
                 print(f"Searching proxies for {len(snps_absent)} SNPs...")
                 ld = find_proxies(
                     snps_absent,
@@ -593,11 +623,17 @@ class Geno:
                     kb=kb,
                     r2=r2,
                     window_snps=window_snps,
-                    threads=self.cpus,
+                    threads=self.cpus
                 )
-                data_prs = apply_proxies(data_prs, ld, searchspace = genetic_snps)
-                check_snp_column(data_prs)
-
+                # Apply proxies if found
+                if isinstance(ld, pd.DataFrame) and not ld.empty:
+                    data_prs = apply_proxies(data_prs, ld, searchspace=genetic_snps)
+                    # Remove duplicates in the SNP column
+                    duplicate_indices = data_prs[data_prs.duplicated(subset=["SNP"], keep="first")].index
+                    n_del = len(duplicate_indices)
+                    if n_del > 0:
+                        data_prs.drop(index=duplicate_indices, inplace=True)
+                        print(f"After proxying, {n_del} SNPs had duplicated IDs and were removed.")
         # Compute PRS
         prs_data = prs_func(data_prs, weighted, path, ram=self.ram, name=self.name)
 
@@ -610,47 +646,42 @@ class Geno:
         return
 
     def set_phenotype(
-        self, data, IID=None, PHENO=None, PHENO_type=None, alternate_control=False
+        self, data, IID=None, PHENO=None, PHENO_type=None, FID=None, alternate_control=False
     ):
         """
         Assign a phenotype dataframe to the .phenotype attribute.
 
         Args:
             data (pd.DataFrame): DataFrame containing individual-level row data with at least an individual IDs column
-                                 and one phenotype column.
+                                and one phenotype column.
             IID (str, optional): Name of the individual IDs column in 'data'. These IDs should
-                                 correspond to the genetic IDs in the FAM file that will be used for association testing.
+                                correspond to the genetic IDs in the FAM file that will be used for association testing.
             PHENO (str, optional): Name of the phenotype column in 'data' which will be used
-                                   as the dependent variable for association tests.
+                                as the dependent variable for association tests.
             PHENO_type (str, optional): If not specified, the function will try to infer if
                                         the phenotype is binary or quantitative. To bypass this,
                                         use "quant" for quantitative or "binary" for binary phenotypes.
                                         Default is None.
+            FID (str, optional): Name of the family ID column in 'data'. If not provided,
+                                FID values will be set to the same as IID values.
             alternate_control (bool, optional): By default, the function assumes that for a binary
                                                 trait, the controls have the most frequent value.
                                                 Set to True if this is not the case. Default is False.
-
-        Returns:
-            None: Sets the .phenotype attribute for the instance.
-
-        Note:
-            This method sets the .phenotype attribute which is essential to perform single-SNP association tests using the association_test method.
-
         """
-        
+
         processed_data, inferred_pheno_type = set_phenotype_func(
-            data, PHENO, PHENO_type, IID, alternate_control
+            data, PHENO, PHENO_type, IID, FID, alternate_control
         )
 
         # Assign the processed data and inferred phenotype type to the .phenotype attribute
-        self.phenotype = (processed_data, inferred_pheno_type)
+        self.phenotype = (processed_data, inferred_pheno_type, PHENO)
 
     def association_test(self, path=None, covar=[], standardize=True):
         """
         Conduct single-SNP association tests against a phenotype.
 
         Args:
-            path (str, optional): Path to a bed/bim/fam set of genetic files.
+            path (str, optional): Path to a bed/bim/fam or pgen/pvar/psam set of genetic files.
                 If files are split by chromosomes, replace the chromosome number with '$'.
                 For instance: path = "ukb_chr$_file". Default is None.
             covar (list, optional): List of columns in the phenotype dataframe to be used
@@ -672,10 +703,16 @@ class Geno:
             raise ValueError(
                 "You first need to set a phenotype using .set_phenotype(data, PHENO, PHENO_type, IID)!"
             )
+        
+        # Ensure that covar does not contain the PHENO column
+        if self.phenotype[2] in covar:
+            print(f"The phenotype column {self.phenotype[2]} is also present in the covariates list and will be removed from the association tests.")
+            covar.remove(self.phenotype[2])
 
         create_tmp() #Make sure temporary folder exists
         
-        # Based on column presents, extract the SNP based names or genomic positions (with preference for positions)
+        n_original = self.data.shape[0]
+        # Based on column presents, extract the SNPs based on names or genomic positions (with preference for positions)
         if "CHR" in self.data.columns and "POS" in self.data.columns:
             print("CHR/POS columns present: SNPs searched based on genomic positions.")
             data = self.update_snpids(path = path)
@@ -694,7 +731,7 @@ class Geno:
             raise ValueError("No SNPs were extracted from the genetic data and the association test can't be run.")
         
         # Perform the association test
-        updated_data = association_test_func(
+        updated_data = association_test_func_plink2(
             data,
             covar,
             standardize,
@@ -703,10 +740,13 @@ class Geno:
             self.phenotype[1],
         )
 
+        n_updated = updated_data.shape[0]
+        print(f"The BETA, SE, P columns of the .data attribute have been updated for {n_updated} SNPs.")
+        if n_updated < n_original:
+            print(f"{n_original - n_updated}({(n_original - n_updated)/n_original*100:.3f}%) SNPs have been removed.")
+
         # Update the instance data
         self.data = updated_data
-
-        print(f"The BETA, SE, P columns of the .data attribute have been updated.")
         return
 
     def query_outcome(
@@ -731,7 +771,7 @@ class Geno:
             proxy (bool, optional): If true, proxies are searched. Default is True.
             reference_panel (str, optional): The reference population to get linkage
                 disequilibrium values and find proxies (only if proxy=True). Acceptable values
-                include "EUR", "SAS", "AFR", "EAS", "AMR" or a path to a specific bed/bim/fam panel.
+                include "EUR", "SAS", "AFR", "EAS", "AMR" or a path to a specific bed/bim/fam or pgen/pvar/psam panel.
                 Default is "EUR".
             kb (int, optional): Width of the genomic window to look for proxies. Default is 5000.
             r2 (float, optional): Minimum linkage disequilibrium value with the main SNP
@@ -770,7 +810,8 @@ class Geno:
         phi=1,
         exposure_name=None,
         outcome_name=None,
-        cpus=-1
+        cpus=-1,
+        odds=False
     ):
         """
         Executes Mendelian Randomization (MR) using the `data_clumped` attribute as exposure data and `MR_data` attribute as outcome data queried using the `query_outcome` method.
@@ -803,6 +844,7 @@ class Geno:
             phi (int, optional): Factor for the bandwidth parameter used in the kernel density estimation of the mode methods
             exposure_name (str, optional): Name of the exposure data (only for display purposes).
             outcome_name (str, optional): Name of the outcome data (only for display purposes).
+            odds (bool, optional): If True, adds an odds ratio column with 95% confidence intervals. Default is False.
 
         Returns:
             pd.DataFrame: A table with MR results.
@@ -820,7 +862,6 @@ class Geno:
             self.MR_data,
             methods,
             action,
-            heterogeneity,
             eaf_threshold,
             nboot,
             penk,
@@ -830,6 +871,17 @@ class Geno:
         )
         
         self.MR_results = (res, df_mr, exposure_name, outcome_name)
+
+        if not heterogeneity:
+            res = res.loc[:,["exposure", "outcome", "method", "nSNP", "b", "se", "pval"]]
+        
+        if odds:
+            # Calculate odds ratios and confidence intervals using .loc
+            res.loc[:,'OR_95CI'] = res.apply(lambda row: 
+                f"{np.exp(row['b']):.3f} ({np.exp(row['b'] - 1.96*row['se']):.3f}-{np.exp(row['b'] + 1.96*row['se']):.3f})" 
+                if not pd.isna(row['b']) and not pd.isna(row['se']) 
+                else np.nan, axis=1)
+                   
         return res
     
     def MR_plot(
@@ -1085,7 +1137,7 @@ class Geno:
             raise ValueError("You must first call query_outcome() before running MR.")
         cpus = self.cpus if cpus == -1 else cpus
 
-        return mrpresso_func(
+        res = mrpresso_func(
             self.MR_data,
             action,
             eaf_threshold,
@@ -1096,6 +1148,10 @@ class Geno:
             cpus,
         )
 
+        self.MRpresso_results = res
+
+        return res
+
     def lift(
         self,
         start="hg19",
@@ -1296,29 +1352,39 @@ class Geno:
             fmt (str, optional): File format. Options: .h5 (default), .csv, .txt. Future: .vcf, .vcf.gz.
             sep (str, optional): Delimiter for .csv and .txt formats. Default is tab.
             header (bool, optional): Save column names for .csv and .txt formats. Default is True.
-
-        Raises:
-            ValueError: If clumped data is requested but data is not clumped.
         """
         save_data(self.data, name=self.name, path=path, fmt=fmt, sep=sep, header=header)
         return
 
-def merge_command_parallel(df_subset, path):
+def merge_command_parallel(df_subset, path, filetype):
     """Helper function of the update_snpids method to update SNP in parallel when genetic data is split by chromosome."""
     chr_number = df_subset.iloc[0]['CHR']
-    bim_path = path.replace("$", str(chr_number)) + ".bim"
-    if not os.path.exists(bim_path):
-        return
-    bim = pd.read_csv(
-        bim_path, sep="\t", names=["CHR", "SNP", "F", "POS", "A1", "A2"]
-    )
-
-    # Convert CHR to string and remove 'chr' prefix if present, then convert to int
-    if str(bim["CHR"][0]).startswith("chr"):
-        bim["CHR"] = bim["CHR"].astype(str).str.replace("^chr", "", regex=True).astype(int)
     
-    bim.drop_duplicates(subset=["CHR", "POS"], keep='first', inplace=True)
+    if filetype == "bed":
+        variant_file = path.replace("$", str(chr_number)) + ".bim"
+        if not os.path.exists(variant_file):
+            return
+        variants = pd.read_csv(
+            variant_file, sep="\t", names=["CHR", "SNP", "F", "POS", "A1", "A2"]
+        )
+    else:  # filetype == "pgen"
+        variant_file = path.replace("$", str(chr_number)) + ".pvar"
+        if not os.path.exists(variant_file):
+            return
+        variants = pd.read_csv(
+            variant_file, sep="\t", comment="#",
+            names=["CHR", "POS", "SNP", "A1", "A2"]
+        )
+
+    # Convert CHR to string and remove 'chr' prefix if present
+    variants["CHR"] = variants["CHR"].astype(str).str.replace("^chr", "", regex=True)
+    # Convert numeric values to int, leave X/Y as strings
+    variants["CHR"] = variants["CHR"].apply(
+        lambda x: int(float(x)) if x.replace(".", "").isdigit() else x
+    )
+    
+    variants.drop_duplicates(subset=["CHR", "POS"], keep='first', inplace=True)
 
     return df_subset.merge(
-        bim[["CHR", "POS", "SNP"]], on=["CHR", "POS"], how="left", suffixes=('', '_new')
+        variants[["CHR", "POS", "SNP"]], on=["CHR", "POS"], how="left", suffixes=('', '_new')
     )
\ No newline at end of file
