@@ -1,46 +1,41 @@
 import pandas as pd
-import numpy as np
 import os, subprocess, re
 from functools import partial
 from concurrent.futures import ProcessPoolExecutor
 
-from .tools import check_bfiles, read_config, write_config, setup_genetic_path, get_plink19_path
+from .tools import check_bfiles, check_pfiles, setup_genetic_path, get_plink_path
 
 
 def prs_func(data, weighted=True, path=None, ram=10000, name=""):
     """
     Compute a PRS (Polygenic Risk Score) using provided SNP-level data. Corresponds to the :meth:`Geno.prs` method
-
-    Args:
-        data (pd.DataFrame): Dataframe containing SNP-level data.
-        weighted (bool, optional): Perform a weighted PRS using BETA column estimates.
-            If False, perform an unweighted PRS (equivalent to BETAs set to 1). Defaults to True.
-        path (str, optional): Path to bed/bim/fam set of genetic files to use for PRS calculation.
-            If not provided, it uses the genetic data extracted with the `.extract_snps` method or attempts to use the path in the config file. Defaults to None.
-        ram (int, optional): RAM memory in MB to be used by plink. Defaults to 10000.
-        name (str, optional): Name used for naming output and intermediate files. Defaults to "".
-
-    Returns:
-        pd.DataFrame: DataFrame containing PRS results.
-
-    Raises:
-        ValueError: If mandatory columns are missing in the data.
-        TypeError: If valid bed/bim/fam files are not found.
-        ValueError: If PRS computation was not successful.
     """
-    
-    # Call extract_snps
-    path = extract_snps_func(data.SNP, name, path)
+    # Get path and filetype
+    path, filetype = setup_genetic_path(path)
 
-    if path == "FAILED":
+    # Call extract_snps
+    extracted_path = extract_snps_func(data.SNP, name, path)
+
+    if extracted_path == "FAILED":
         raise ValueError("No SNPs were extracted from the genetic data and the PRS can't be computed.")
 
+    # Write processed data to file and run plink on it
+    data = data[["SNP", "EA", "BETA"]]
+    data_path = os.path.join("tmp_GENAL", f"{name}_to_prs.txt")
+    output_path = os.path.join("tmp_GENAL", f"{name}_prs")
+
+    data.to_csv(data_path, sep="\t", index=False, header=True)
+    
+    # We can use --pfile since extract_snps now creates pgen files
+    plink_command = f"{get_plink_path()} --memory {ram} --pfile {extracted_path} \
+                     --score {data_path} 1 2 3 header --out {output_path} --allow-no-sex"
+
     # Set BETA values to 1 if unweighted PRS is required
     if not weighted:
         data["BETA"] = 1
-        print(f"Computing an unweighted PRS using {path} data.")
+        print(f"Computing an unweighted PRS using {extracted_path} data.")
     else:
-        print(f"Computing a weighted PRS using {path} data.")
+        print(f"Computing a weighted PRS using {extracted_path} data.")
 
     # Check for empty dataframe
     n_snps = data.shape[0]
@@ -49,14 +44,6 @@ def prs_func(data, weighted=True, path=None, ram=10000, name=""):
             "No SNPs remain for the polygenic risk score (PRS) calculation."
         )
 
-    # Write processed data to file and run plink on it
-    data = data[["SNP", "EA", "BETA"]]
-    data_path = os.path.join("tmp_GENAL", f"{name}_to_prs.txt")
-    output_path = os.path.join("tmp_GENAL", f"{name}_prs")
-
-    data.to_csv(data_path, sep="\t", index=False, header=True)
-    plink_command = f"{get_plink19_path()} --memory {ram} --bfile {path} \
-                     --score {data_path} 1 2 3 header --out {output_path} --allow-no-sex"
     try:
         output = subprocess.run(
             plink_command, shell=True, capture_output=True, text=True, check=True
@@ -68,22 +55,23 @@ def prs_func(data, weighted=True, path=None, ram=10000, name=""):
         raise ValueError("PLINK command failed. Check the error messages above for details.")
 
     # Read and process PRS results
-    prs_file = output_path + ".profile"
+    prs_file = output_path + ".sscore"
     log_file = output_path + ".log"
     if os.path.isfile(prs_file): #If the profile file exists: PRS was successful
         #Extracts the number of SNPs used for the PRS computation
-        pattern = r'--score: (\d+) valid predictor'
-        with open(log_file, 'r') as file:
-            for line in file:
-                match = re.search(pattern, line)
-                if match:
-                    n_predictors = int(match.group(1))
-        #Return scores
-        print(
+        log_content = open(log_file).read()
+        match = re.search(r'--score: (\d+) variant[s] processed', log_content)
+        if match:
+            n_predictors = int(match.group(1))
+            print(
             f"The PRS computation was successful and used {n_predictors}/{n_snps} ({n_predictors/n_snps*100:.3f}%) SNPs."
-        )
+            )
+        else:
+            print("Could not extract the number of SNPs used for the PRS computation.")
+        #Return scores
         df_score = pd.read_csv(prs_file, sep="\s+")
-        return df_score[["FID", "IID", "SCORE"]]
+        df_score.rename(columns={"#FID": "FID"}, inplace=True)
+        return df_score
     else:
         print(output.stdout)
         raise ValueError(
@@ -96,7 +84,7 @@ def extract_snps_func(snp_list, name, path=None):
     Extracts a list of SNPs from the given path. This function corresponds to the following Geno method: :meth:`Geno.extract_snps`.
 
     Args:
-        snp_list (List[str]): List of SNPs to extract.
+        snp_list (pd.Series): Series of SNPs to extract.
         name (str): Name prefix for the output files.
         path (str, optional): Path to the dataset. Defaults to the path from the configuration.
 
@@ -106,30 +94,35 @@ def extract_snps_func(snp_list, name, path=None):
     Raises:
         TypeError: Raises an error when no valid path is saved or when there's an incorrect format in the provided path.
     """
-    path = setup_genetic_path(path)
+    # Check if snp_list is empty Series
+    if snp_list.empty:
+        print("The provided SNP list is empty.")
+        return "FAILED"
+    
+    # Get path and filetype
+    path, filetype = setup_genetic_path(path)
 
     snp_list, snp_list_path, nrow = prepare_snp_list(snp_list, name)
 
-    filetype = "split" if "$" in path else "combined"
+    filetype_split = "split" if "$" in path else "combined"
 
     output_path = os.path.join("tmp_GENAL", f"{name}_allchr")
-    if filetype == "split":
+    if filetype_split == "split":
         merge_command, bedlist_path = extract_snps_from_split_data(
-            name, path, output_path, snp_list_path
+            name, path, output_path, snp_list_path, filetype
         )
         handle_multiallelic_variants(name, merge_command, bedlist_path)
     else:
-        extract_snps_from_combined_data(name, path, output_path, snp_list_path)
+        extract_snps_from_combined_data(name, path, output_path, snp_list_path, filetype)
 
     #Check that at least 1 variant has been extracted. If not, return "FAILED" to warn downstream functions (prs, association_test)
     log_path = output_path + ".log"
     with open(log_path, 'r') as log_file:
-        if "No variants remaining" in log_file.read():
+        if "0 variants remaining" in log_file.read():
             print("None of the provided SNPs were found in the genetic data.")
             return "FAILED"
-
         else:
-            print(f"Created bed/bim/fam fileset with extracted SNPs: {output_path}")
+            print(f"Created pgen/pvar/psam fileset with extracted SNPs: {output_path}")
             # Report SNPs not found
             report_snps_not_found(nrow, name)
 
@@ -146,7 +139,7 @@ def prepare_snp_list(snp_list, name):
     return snp_list, snp_list_path, nrow
 
 
-def extract_command_parallel(task_id, name, path, snp_list_path):
+def extract_command_parallel(task_id, name, path, snp_list_path, filetype):
     """
     Helper function to run SNP extraction in parallel for different chromosomes.
     Args:
@@ -154,35 +147,48 @@ def extract_command_parallel(task_id, name, path, snp_list_path):
         name (str): Name prefix for the output files.
         path (str): Path to the data set.
         snp_list_path (str): Path to the list of SNPs to extract.
+        filetype (str): Type of genetic files ("bed" or "pgen")
     Returns:
-        int: Returns the task_id if no valid bed/bim/fam files are found.
+        int: Returns the task_id if no valid files are found.
     """
-    bfile_path = path.replace("$", str(task_id))
+    input_path = path.replace("$", str(task_id))
 
-    if not check_bfiles(bfile_path):
+    # Check if files exist based on filetype
+    if filetype == "bed" and not check_bfiles(input_path):
+        return task_id
+    elif filetype == "pgen" and not check_pfiles(input_path):
         return task_id
 
     output_path = os.path.join("tmp_GENAL", f"{name}_extract_chr{task_id}")
-    command = f"{get_plink19_path()} --bfile {bfile_path} --extract {snp_list_path} --make-bed --out {output_path}"
+    
+    # Build command based on filetype
+    base_cmd = f"{get_plink_path()}"
+    if filetype == "bed":
+        base_cmd += f" --bfile {input_path}"
+    else:  # pgen
+        base_cmd += f" --pfile {input_path}"
+        
+    command = f"{base_cmd} --extract {snp_list_path} --make-pgen --out {output_path}"
+    
     subprocess.run(
         command, shell=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL
     )
 
 
-def create_bedlist(bedlist, output_name, not_found):
+def create_bedlist(bedlist_path, output_name, not_found):
     """
     Creates a bedlist file for SNP extraction.
     Args:
-        bedlist (str): Path to save the bedlist file.
+        bedlist_path (str): Path to save the bedlist file.
         output_name (str): Base name for the output files.
-        not_found (List[int]): List of chromosome numbers for which no bed/bim/fam files were found.
+        not_found (List[int]): List of chromosome numbers for which no files were found.
     """
-    with open(bedlist, "w+") as bedlist_file:
+    with open(bedlist_path, "w+") as bedlist_file:
         n_lines = 0
         for i in range(1, 23):
             if i in not_found:
-                print(f"bed/bim/fam files not found for chr{i}.")
-            elif check_bfiles(f"{output_name}_chr{i}"):
+                print(f"pgen/pvar/psam files not found for chr{i}.")
+            elif check_pfiles(f"{output_name}_chr{i}"):
                 bedlist_file.write(f"{output_name}_chr{i}\n")
                 n_lines += 1
                 print(f"SNPs extracted for chr{i}.")
@@ -191,12 +197,16 @@ def create_bedlist(bedlist, output_name, not_found):
     return n_lines
 
 
-def extract_snps_from_split_data(name, path, output_path, snp_list_path):
+def extract_snps_from_split_data(name, path, output_path, snp_list_path, filetype):
     """Extract SNPs from data split by chromosome."""
     print("Extracting SNPs for each chromosome...")
     num_tasks = 22
     partial_extract_command_parallel = partial(
-        extract_command_parallel, name=name, path=path, snp_list_path=snp_list_path
+        extract_command_parallel, 
+        name=name, 
+        path=path, 
+        snp_list_path=snp_list_path,
+        filetype=filetype
     )  # Wrapper function
     with ProcessPoolExecutor() as executor:
         not_found = list(
@@ -213,29 +223,33 @@ def extract_snps_from_split_data(name, path, output_path, snp_list_path):
         raise Warning("No SNPs were extracted from any chromosome.")
 
     print("Merging SNPs extracted from each chromosome...")
-    merge_command = f"{get_plink19_path()} --merge-list {bedlist_path} --make-bed --out {output_path}"
+    merge_command = f"{get_plink_path()} --pmerge-list {bedlist_path} pfile --out {output_path}"
     subprocess.run(
         merge_command, shell=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL
     )
 
     return merge_command, bedlist_path
 
-
+# TODO: Check if this function is still needed with plink2
 def handle_multiallelic_variants(name, merge_command, bedlist_path):
     """Handle multiallelic variants detected during merging."""
 
     def remove_multiallelic():
-        snps_to_exclude = pd.read_csv(
-            os.path.join("tmp_GENAL", f"{name}_allchr-merge.missnp"), header=None
-        )
+        missnp_path = os.path.join(
+            "tmp_GENAL", f"{name}_allchr.vmiss"
+            )
+        if not os.path.exists(missnp_path):
+            return 0
+            
+        snps_to_exclude = pd.read_csv(missnp_path, header=None)
         for i in range(1, 23):
-            bim_path = os.path.join("tmp_GENAL", f"{name}_extract_chr{i}.bim")
-            if not os.path.isfile(bim_path):
+            pvar_path = os.path.join("tmp_GENAL", f"{name}_extract_chr{i}.pvar")
+            if not os.path.isfile(pvar_path):
                 continue
-            bim = pd.read_csv(bim_path, sep="\t", header=None)
+            pvar = pd.read_csv(pvar_path, sep="\t", header=None)
             # If no SNPs would be left for this chr: remove corresponding bedlist line
-            n_to_exclude = len(set(bim[1]).intersection(set(snps_to_exclude[0])))
-            if n_to_exclude == len(set(bim[1])):
+            n_to_exclude = len(set(pvar[2]).intersection(set(snps_to_exclude[0])))
+            if n_to_exclude == len(set(pvar[2])):
                 print(f"No SNPs remaining for chromosome {i}.")
                 tmp_filename = os.path.join("tmp_GENAL", "tmp_multiallelic")
                 with open(bedlist_path, "r") as file, open(
@@ -243,7 +257,7 @@ def handle_multiallelic_variants(name, merge_command, bedlist_path):
                 ) as temp_file:
                     output_name = os.path.join("tmp_GENAL", f"{name}_extract")
                     line_to_exclude = f"{output_name}_chr{i}\n"
-                    for current_line_number, line in enumerate(file, start=1):
+                    for line in file:
                         if line != line_to_exclude:
                             temp_file.write(line)
                 # Replace the original file with the temporary file
@@ -251,12 +265,8 @@ def handle_multiallelic_variants(name, merge_command, bedlist_path):
 
             # If there is at least one multiallelic SNP for this chr
             elif n_to_exclude > 0:
-                bfile_path = os.path.join("tmp_GENAL", f"{name}_extract_chr{i}")
-                missnp_path = os.path.join(
-                    "tmp_GENAL", f"{name}_allchr-merge.missnp"
-                )
-                output_path = os.path.join("tmp_GENAL", f"{name}_extract_chr{i}")
-                command = f"{get_plink19_path()} --bfile {bfile_path} --exclude {missnp_path} --make-bed --out {output_path}"
+                pfile_path = os.path.join("tmp_GENAL", f"{name}_extract_chr{i}")
+                command = f"{get_plink_path()} --pfile {pfile_path} --exclude {missnp_path} --make-pgen --out {pfile_path}"
                 subprocess.run(
                     command,
                     shell=True,
@@ -266,7 +276,7 @@ def handle_multiallelic_variants(name, merge_command, bedlist_path):
         return len(snps_to_exclude)
 
     log_content = open(os.path.join("tmp_GENAL", f"{name}_allchr.log")).read()
-    if "with 3+ alleles present" in log_content:
+    if "Error: Multiple" in log_content:
         print("Multiallelic variants detected in the genetic files: removing them before merging.")
         n_multiallelic = remove_multiallelic()
         print(f"Reattempting the merge after exclusion of {n_multiallelic} multiallelic variants.")
@@ -277,31 +287,26 @@ def handle_multiallelic_variants(name, merge_command, bedlist_path):
             stderr=subprocess.DEVNULL,
         )
 
-        ## In rare cases, plink fails to identify all the multiallelic SNPs with the first pass.
-        log_content = open(os.path.join("tmp_GENAL", f"{name}_allchr.log")).read()
-        if "with 3+ alleles present" in log_content:
-            print("Multiallelic variants still detected: removing them before merging.")
-            remove_multiallelic()
-            print("Reattempting the merge after deletion of multiallelic variants.")
-            subprocess.run(
-                merge_command,
-                shell=True,
-                stdout=subprocess.DEVNULL,
-                stderr=subprocess.DEVNULL,
-            )
 
-
-def extract_snps_from_combined_data(name, path, output_path, snp_list_path):
+def extract_snps_from_combined_data(name, path, output_path, snp_list_path, filetype):
     """Extract SNPs from combined data."""
     print("Extracting SNPs...")
-    extract_command = f"{get_plink19_path()} --bfile {path} --extract {snp_list_path} --make-bed --out {output_path}"
+    
+    # Build command based on filetype
+    base_cmd = f"{get_plink_path()}"
+    if filetype == "bed":
+        base_cmd += f" --bfile {path}"
+    else:  # pgen
+        base_cmd += f" --pfile {path}"
+        
+    extract_command = f"{base_cmd} --extract {snp_list_path} --make-pgen --out {output_path}"
+    
     subprocess.run(
         extract_command,
         shell=True,
         stdout=subprocess.DEVNULL,
         stderr=subprocess.DEVNULL,
     )
-    
 
 
 def report_snps_not_found(nrow, name):
@@ -311,8 +316,8 @@ def report_snps_not_found(nrow, name):
         with open(filepath, "r") as file:
             return sum(1 for line in file)
 
-    file_path = os.path.join("tmp_GENAL", f"{name}_allchr.bim")
-    extracted_snps_count = count_lines(file_path)
+    file_path = os.path.join("tmp_GENAL", f"{name}_allchr.pvar")
+    extracted_snps_count = count_lines(file_path)-1 #pvar files include column names
     delta_nrow = nrow - extracted_snps_count
     if delta_nrow > 0:
         print(
