@@ -145,7 +145,7 @@ def fill_ea_nea(data, reference_panel_df):
     )
     if perc_missing > 50:
         warnings.warn(
-            f"The EA (Effect Allele) and NEA (Non-Effect Allele) for many SNPs could not been found. Make sure the CHR/POS coordinates are in build GRCh37 (hg19). If not, you can first use the .lift() method to lift them. For instance: .lift(start='hg38', end='hg19', replace=True) if they are in build GRCh38 (hg38)."
+            f"The EA (Effect Allele) and NEA (Non-Effect Allele) for many SNPs could not been found. Make sure the CHR/POS coordinates are in the same build as the reference panel (GRCh37 (hg19) for the default one). If not, you can first use the .lift() method to lift them. For instance: .lift(start='hg38', end='hg19', replace=True) if they are in build GRCh38 (hg38)."
         )
     return data
 
@@ -167,7 +167,7 @@ def fill_nea(data, reference_panel_df):
     )
     if perc_missing > 50:
         warnings.warn(
-            f"The NEA (Non Effect Allele) for many SNPs could not been found. Make sure the CHR/POS coordinates are in build GRCh37 (hg19). If not, you can first use the .lift() method to lift them. For instance: .lift(start='hg38', end='hg19', replace=True) if they are in build GRCh38 (hg38)."
+            f"The NEA (Non Effect Allele) for many SNPs could not been found. Make sure the CHR/POS coordinates are in the same build as the reference panel (GRCh37 (hg19) for the default one). If not, you can first use the .lift() method to lift them. For instance: .lift(start='hg38', end='hg19', replace=True) if they are in build GRCh38 (hg38)."
         )
     return data
 
@@ -184,7 +184,7 @@ def fill_coordinates_func(data, reference_panel_df):
     data["CHR"] = data["CHR"].astype("Int32")
     data["POS"] = data["POS"].astype("Int32")
     print(
-        f"The coordinates columns (CHR for chromosome and POS for position) have been created following build GRCh37 (hg19). {n_missing}({n_missing/data.shape[0]*100:.3f}%) SNPs were not found in the reference data and their values set to nan."
+        f"The coordinates columns (CHR for chromosome and POS for position) have been created. {n_missing}({n_missing/data.shape[0]*100:.3f}%) SNPs were not found in the reference data and their values set to nan."
     )
     return data
 
@@ -232,7 +232,7 @@ def fill_snpids_func(data, reference_panel_df):
         
     if perc_missing > 50:
         warnings.warn(
-            f"The SNPid for many SNPs could not been found. Make sure the CHR/POS coordinates are in build GRCh37 (hg19). If not, you can first use the .lift() method to lift them. For instance: .lift(start='hg38', end='hg19', replace=True) if they are in build GRCh38 (hg38)."
+            f"The SNPid for many SNPs could not been found. Make sure the CHR/POS coordinates are in the same build as the reference panel (GRCh37 (hg19) for the default one). If not, you can first use the .lift() method to lift them. For instance: .lift(start='hg38', end='hg19', replace=True) if they are in build GRCh38 (hg38)."
         )
 
     return data
@@ -257,6 +257,7 @@ def adjust_column_names(data, CHR, POS, SNP, EA, NEA, BETA, SE, P, EAF, keep_col
     Rename columns to the standard names making sure that there are no duplicated names.
     Delete other columns if keep_columns=False, keep them if True.
     """
+
     # Check keep_columns argument
     if not isinstance(keep_columns, bool):
         raise TypeError(f"{keep_columns} only accepts values: True or False.")
@@ -275,11 +276,13 @@ def adjust_column_names(data, CHR, POS, SNP, EA, NEA, BETA, SE, P, EAF, keep_col
     for key, value in rename_dict.items():
         if key != value and key not in data.columns:
             raise TypeError(f"Column {key} is not found in the dataframe.")
+
     if not keep_columns:
-        cols_to_keep = [CHR, POS, SNP, EA, NEA, BETA, SE, P, EAF]
-        cols_to_drop = [col for col in data.columns if col not in cols_to_keep]
+        cols_to_drop = [col for col in data.columns if col not in rename_dict.keys()]
         data.drop(columns=cols_to_drop, inplace=True)
+
     data.rename(columns=rename_dict, inplace=True)
+
     # Check duplicated column names
     column_counts = Counter(data.columns)
     duplicated_columns = [
@@ -291,12 +294,15 @@ def adjust_column_names(data, CHR, POS, SNP, EA, NEA, BETA, SE, P, EAF, keep_col
         raise ValueError(
             f"After adjusting the column names, the resulting dataframe has duplicated columns. Make sure your dataframe does not have a different column named {duplicated_columns}."
         )
+    
+    # Print columns found
+    print(f"The following columns were found: {list(set(rename_dict.values()) & set(data.columns.to_list()))}")
+    
     return data
 
 
 def check_arguments(
     preprocessing,
-    reference_panel,
     effect_column,
     fill_snpids,
     fill_coordinates,
@@ -415,4 +421,4 @@ def Combine_Geno(Gs):
 
     C = C.reset_index(drop=True)
 
-    return Geno(C)
\ No newline at end of file
+    return Geno(C)
